﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Text;
using Vevo.Domain;
using Vevo.Domain.Base;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.EmailTemplates;
using Vevo.Shared.DataAccess;
using Vevo.Base.Data;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class EmailTemplateDetailRepositoryNoCache : IEmailTemplateDetailRepository
    {
        #region Private

        private const string AllFields =
            "EmailTemplateDetail.EmailTemplateDetailID, EmailTemplateDetailName, Description, StoreID, CultureID, Subject, Body ";
        private const string AllMainFields = "EmailTemplateDetailID, EmailTemplateDetailName, Description, StoreID ";
        private const string AllLocaleFields = "EmailTemplateDetailID, CultureID, Subject, Body ";

        #endregion

        #region Private Method

        private List<ILocale> ConvertLocaleTableToList( DataTable localeTable )
        {
            List<ILocale> list = new List<ILocale>();

            foreach (DataRow row in localeTable.Rows)
                list.Add( CreateOneLocaleFromRow( row ) );

            return list;
        }

        private EmailTemplateDetailLocale CreateOneLocaleFromRow( DataRow row )
        {
            EmailTemplateDetailLocale locale = new EmailTemplateDetailLocale();
            locale.CultureID = row["CultureID"].ToString();
            locale.Subject = row["Subject"].ToString();
            locale.Body = row["Body"].ToString();

            return locale;
        }

        private void DeleteLocales( string emailTemplateDetailID )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE FROM EmailTemplateDetailLocale " +
                "WHERE EmailTemplateDetailID = @EmailTemplateDetailID; ",
                DataAccess.CreateParameterString( emailTemplateDetailID ) );
        }

        private void InsertLocales( EmailTemplateDetail emailTemplateDetail )
        {
            foreach (EmailTemplateDetailLocale locale in emailTemplateDetail.Locales)
            {
                DataAccess.ExecuteNonQuery(
                "INSERT INTO EmailTemplateDetailLocale ( " + AllLocaleFields + " ) " +
                "VALUES ( @EmailTemplateDetailID, @CultureID, @Subject, @Body ); " +
                "SELECT @@Identity; ",
                DataAccess.CreateParameterString( emailTemplateDetail.EmailTemplateDetailID ),
                DataAccess.CreateParameterString( locale.CultureID ),
                DataAccess.CreateParameterString( locale.Subject ),
                DataAccess.CreateParameterString( locale.Body )
                );
            }
        }

        private EmailTemplateDetail Create( EmailTemplateDetail emailTemplateDetail )
        {
            string emailTemplateDetailID = DataAccess.ExecuteScalar(
               "INSERT INTO EmailTemplateDetail ( EmailTemplateDetailName, Description, StoreID ) " +
               "VALUES ( @EmailTemplateDetailName, @Description, @StoreID ); " +
               "SELECT @@Identity; ",
               DataAccess.CreateParameterString( emailTemplateDetail.EmailTemplateDetailName ),
               DataAccess.CreateParameterString( emailTemplateDetail.Description ),
               DataAccess.CreateParameterString( emailTemplateDetail.StoreID )
               );

            emailTemplateDetail.EmailTemplateDetailID = emailTemplateDetailID;

            InsertLocales( emailTemplateDetail );

            return emailTemplateDetail;
        }

        private EmailTemplateDetail Update( EmailTemplateDetail emailTemplateDetail )
        {
            DataAccess.ExecuteNonQuery( "UPDATE EmailTemplateDetail " +
                "SET EmailTemplateDetailName = @EmailTemplateDetailName, " +
                "Description=@Description, StoreID=@StoreID " +
                "WHERE EmailTemplateDetailID = @EmailTemplateDetailID; ",
               DataAccess.CreateParameterString( emailTemplateDetail.EmailTemplateDetailName ),
               DataAccess.CreateParameterString( emailTemplateDetail.Description ),
               DataAccess.CreateParameterString( emailTemplateDetail.StoreID ),
               DataAccess.CreateParameterString( emailTemplateDetail.EmailTemplateDetailID ) );

            DeleteLocales( emailTemplateDetail.EmailTemplateDetailID );
            InsertLocales( emailTemplateDetail );

            return emailTemplateDetail;
        }

        private EmailTemplateDetail CreateOneFromRow( Culture culture, DataRow row )
        {
            EmailTemplateDetail emailTemplateDetail = new EmailTemplateDetail( culture );

            emailTemplateDetail.EmailTemplateDetailID = row["EmailTemplateDetailID"].ToString();
            emailTemplateDetail.EmailTemplateDetailName = row["EmailTemplateDetailName"].ToString();
            emailTemplateDetail.Description = row["Description"].ToString();
            emailTemplateDetail.StoreID = row["StoreID"].ToString();
            emailTemplateDetail.Subject = row["Subject"].ToString();
            emailTemplateDetail.Body = row["Body"].ToString();

            return emailTemplateDetail;
        }

        private IList<EmailTemplateDetail> ConvertTableToList( Culture culture, DataTable table )
        {
            return ConvertTableToList( culture, table, 0, table.Rows.Count );
        }

        private IList<EmailTemplateDetail> ConvertTableToList( Culture culture, DataTable table, int startIndex, int endIndex )
        {
            IList<EmailTemplateDetail> list = new List<EmailTemplateDetail>();

            for (int i = startIndex; i <= endIndex && i < table.Rows.Count; i++)
                list.Add( CreateOneFromRow( culture, table.Rows[i] ) );

            return list;
        }

        private string GetCultureID( Culture culture )
        {
            return culture.CultureID;
        }

        private void CopyEmailTemplateDetailLocales( EmailTemplateDetail emailTemplateDetail, IList<ILocale> locales )
        {
            foreach (EmailTemplateDetailLocale locale in locales)
            {
                emailTemplateDetail.Locales.Add( locale );
            }
        }

        #endregion

        #region Public Method

        public IList<ILocale> GetLocales( string emailTemplateDetailID )
        {
            DataTable localeTable = DataAccess.ExecuteSelect(
               "SELECT " + AllLocaleFields +
               "FROM EmailTemplateDetailLocale " +
               "WHERE EmailTemplateDetailID = @EmailTemplateDetailID; ",
               DataAccess.CreateParameterString( emailTemplateDetailID ) );

            return ConvertLocaleTableToList( localeTable );
        }

        public EmailTemplateDetail Save( EmailTemplateDetail emailTemplateDetail )
        {
            EmailTemplateDetail result;

            using (DataTransactionScope transaction = new DataTransactionScope())
            {
                if (emailTemplateDetail.EmailTemplateDetailID == "0")
                    result = Create( emailTemplateDetail );
                else
                    result = Update( emailTemplateDetail );

                transaction.Complete();
            }

            return result;
        }

        public EmailTemplateDetail GetOne( Culture culture, string emailTemplateDetailID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllFields +
                "FROM EmailTemplateDetail " +
                "LEFT JOIN (SELECT * FROM EmailTemplateDetailLocale WHERE CultureID = @CultureID) L " +
                "ON EmailTemplateDetail.EmailTemplateDetailID = L.EmailTemplateDetailID " +
                "WHERE EmailTemplateDetail.EmailTemplateDetailID = @EmailTemplateDetailID; ",
                DataAccess.CreateParameterString( RepositoryUtilities.GetCultureID( culture ) ),
                DataAccess.CreateParameterString( emailTemplateDetailID )
                );

            if (table.Rows.Count > 0)
            {
                EmailTemplateDetail emailTemplateDetail = CreateOneFromRow( culture, table.Rows[0] );
                CopyEmailTemplateDetailLocales( emailTemplateDetail, GetLocales( emailTemplateDetailID ) );
                return emailTemplateDetail;
            }
            else
                return EmailTemplateDetail.Null;
        }

        public string GetIDByNameAndStoreID( Culture culture, string emailTemplateDetailName, string storeID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllFields +
                "FROM EmailTemplateDetail " +
                "LEFT JOIN (SELECT * FROM EmailTemplateDetailLocale WHERE CultureID = @CultureID) L " +
                "ON EmailTemplateDetail.EmailTemplateDetailID = L.EmailTemplateDetailID " +
                "WHERE EmailTemplateDetail.EmailTemplateDetailName = @EmailTemplateDetailName " +
                "AND StoreID = @StoreID; ",
                DataAccess.CreateParameterString( RepositoryUtilities.GetCultureID( culture ) ),
                DataAccess.CreateParameterString( emailTemplateDetailName ),
                DataAccess.CreateParameterString( storeID )
                );

            if (table.Rows.Count > 0)
                return table.Rows[0]["EmailTemplateDetailID"].ToString();
            else
                return "0";
        }

        public void Delete( string emailTemplateDetailID )
        {
            using (DataTransactionScope transaction = new DataTransactionScope())
            {
                DataAccess.ExecuteNonQuery(
                "DELETE " +
                "FROM EmailTemplateDetail " +
                "WHERE EmailTemplateDetailID = @EmailTemplateDetailID; ",
                DataAccess.CreateParameterString( emailTemplateDetailID ) );

                DeleteLocales( emailTemplateDetailID );
                transaction.Complete();
            }
        }

        public void DeleteLocalesByCultureID( string cultureID )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE " +
                "FROM EmailTemplateDetailLocale " +
                "WHERE CultureID = @CultureID ;",
                DataAccess.CreateParameterString( cultureID ) );
        }

        public IList<EmailTemplateDetail> GetAll( Culture culture, string sortBy, string storeID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllFields +
                "FROM EmailTemplateDetail " +
                "LEFT JOIN (SELECT * FROM EmailTemplateDetailLocale WHERE CultureID = @CultureID) L " +
                "ON EmailTemplateDetail.EmailTemplateDetailID = L.EmailTemplateDetailID " +
                "Where StoreID = @StoreID " +
                "ORDER BY " + sortBy + "; ",
                DataAccess.CreateParameterString( RepositoryUtilities.GetCultureID( culture ) ),
                DataAccess.CreateParameterString( storeID )
                );

            return ConvertTableToList( culture, table );
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllFields +
                "FROM EmailTemplateDetail INNER JOIN EmailTemplateDetailLocale L " +
                "ON EmailTemplateDetail.EmailTemplateDetailID = L.EmailTemplateDetailID WHERE  1 = 0; "
                );

            return RepositoryUtilities.ConvertTableToSchemaItemList( table );
        }

        #endregion
    }
}
