using System;
using System.Collections.Generic;
using System.Text;
using System.Web.UI;
using Vevo.Base.Data.Caching;
using Vevo.Domain;
using Vevo.Domain.Base;
using Vevo.Domain.Contents;
using Vevo.Domain.DataInterfaces;
using Vevo.Shared.Utilities;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class NewsRepository : INewsRepository
    {
        #region Private

        private const string RepositoryName = "NewsRepository";

        private const string GetOneSignature = "GetOne";
        private const string GetLocalesSignature = "GetLocales";
        private const string GetAllSignature = "GetAll";
        private const string GetHotNewsSignature = "GetHotNews";
        private const string GetTableSchemaSignature = "GetTableSchema";
        private const string SearchNewsSignature = "SearchNews";
        private const string GetNewsIDFromUrlNameSignature = "GetNewsIDFromUrlName";
        private const string GetLatestNewsSignature = "GetLatestNews";

        private DataCache _dataCache = new DataCache( RepositoryName );
        private ICacheDependencyUtility _cacheDependency = new CacheDependencyUtility( new string[] { "NewsLocale", "News" } );

        private NewsRepositoryNoCache _repository = new NewsRepositoryNoCache();

        #endregion

        #region Constructors

        public NewsRepository()
        {
        }

        public NewsRepository( ICacheDependencyUtility cacheDependency )
        {
            _cacheDependency = cacheDependency;
        }

        #endregion

        #region Public Methods

        public News Save( News news )
        {
            // Need to write to database first before clearing cache.
            // Otherwise, cache may remember incorrect values before saving.
            News result = _repository.Save( news );
            _dataCache.ClearRepositoryCache();
            return result;
        }

        public News GetOne( Culture culture, string newsID )
        {
            News result = (News) _dataCache.GetValue( GetOneSignature, _cacheDependency, culture.CultureID, newsID );

            if (result == null)
            {
                result = _repository.GetOne( culture, newsID );
                _dataCache.SetValue( GetOneSignature, result, culture.CultureID, newsID );
            }

            return result;
        }

        public void Delete( string newsID )
        {
            _repository.Delete( newsID );
            _dataCache.ClearRepositoryCache();
        }

        public void DeleteLocalesByCultureID( string cultureID )
        {            
            _repository.DeleteLocalesByCultureID( cultureID );
            _dataCache.ClearRepositoryCache();
        }

        public void DeleteByStoreID( string storeID )
        {
            _repository.DeleteByStoreID( storeID );
            _dataCache.ClearRepositoryCache();
        }

        public IList<ILocale> GetLocales( string newsID )
        {
            IList<ILocale> list = (IList<ILocale>) _dataCache.GetValue( GetLocalesSignature, _cacheDependency, newsID );

            if (list == null)
            {
                list = _repository.GetLocales( newsID );
                _dataCache.SetValue( GetLocalesSignature, list, newsID );
            }

            return list;
        }

        public IList<News> GetAll( Culture culture, string sortBy,string storeID )
        {
            IList<News> list = (IList<News>) _dataCache.GetValue( GetAllSignature, _cacheDependency, sortBy,storeID );

            if (list == null)
            {
                list = _repository.GetAll( culture, sortBy,storeID );
                _dataCache.SetValue( GetAllSignature, list, sortBy, storeID );
            }
            return list;
        }
        public IList<News> GetAll( Culture culture, string sortBy, string storeID, BoolFilter isEnabled )
        {
            IList<News> list = ( IList<News> ) _dataCache.GetValue( GetAllSignature, _cacheDependency, sortBy, storeID, isEnabled.ToString() );
            if ( list == null )
            {
                list = _repository.GetAll( culture, sortBy, storeID, isEnabled );
                _dataCache.SetValue( GetAllSignature, list, sortBy, storeID, isEnabled.ToString() );
            }

            return list;
        }

        public IList<News> GetHotNews( Culture culture, BoolFilter isHotNews, string storeID )
        {
            IList<News> list = (IList<News>) _dataCache.GetValue(
                GetAllSignature,
                _cacheDependency,
                culture.CultureID,
                isHotNews.ToString(),
                storeID
                );

            if (list == null)
            {
                list = _repository.GetHotNews( culture, isHotNews,storeID );
                _dataCache.SetValue( GetAllSignature, list, culture.CultureID, isHotNews.ToString(),storeID );
            }

            return list;
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            IList<TableSchemaItem> list = (IList<TableSchemaItem>) _dataCache.GetValue( GetTableSchemaSignature, _cacheDependency );

            if (list == null)
            {
                list = _repository.GetTableSchema();
                _dataCache.SetValue(
                    GetTableSchemaSignature,
                    list
                    );
            }

            return list;
        }

        public IList<News> SearchNews(
            Culture culture,
            string sortBy,
            SearchFilter searchFilter,
            int startIndex,
            int endIndex,
            out int howManyItems,
            string storeID,
            BoolFilter isEnabled)
        {
            howManyItems = 0;
            Pair result = (Pair) _dataCache.GetValue(
                SearchNewsSignature,
                _cacheDependency,
                culture.CultureID,
                sortBy,
                searchFilter.ToString(),
                startIndex,
                endIndex,
                storeID,
                isEnabled.ToString()
                );

            if (result == null)
            {
                result = new Pair();
                result.First = _repository.SearchNews(
                    culture, sortBy, searchFilter, startIndex, endIndex, out howManyItems, storeID, isEnabled );
                result.Second = howManyItems;
                _dataCache.SetValue(
                    SearchNewsSignature,
                    result,
                    culture.CultureID,
                    sortBy,
                    searchFilter.ToString(),
                    startIndex,
                    endIndex ,
                    storeID,
                    isEnabled.ToString()
                    );
            }

            howManyItems = ConvertUtilities.ToInt32( result.Second );
            return (IList<News>) result.First;
        }

        public string GetNewsIDFromUrlName( string urlName )
        {
            object newsID = _dataCache.GetValue( GetNewsIDFromUrlNameSignature, _cacheDependency, urlName );
            if ( newsID == null )
            {
                newsID = _repository.GetNewsIDFromUrlName( urlName );
                _dataCache.SetValue( GetNewsIDFromUrlNameSignature, newsID, urlName );
            }
            return newsID.ToString();
        }

        public IList<News> GetLatestNews( string latestNews, Culture culture, string storeID )
        {
            IList<News> list = (IList<News>) _dataCache.GetValue(
                GetLatestNewsSignature,
                _cacheDependency,
                latestNews,
                culture.CultureID,
                storeID
                );

            if (list == null)
            {
                list = _repository.GetLatestNews( latestNews, culture, storeID );
                _dataCache.SetValue( GetLatestNewsSignature, list, latestNews, culture.CultureID, storeID );
            }

            return list;
        }
        #endregion
    }
}
