using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using Vevo.Domain.Products;

namespace Vevo.Domain.Base
{
    [Serializable]
    public class LocaleCollection<T> : IList<T>, ICloneable
        where T : ILocale, new()        // T must have ILocale interface; T must have a default constructor.
    {
        #region Protected

        protected bool _loaded = false;
        protected ILocaleLoader _localeLoader;
        protected IList<T> InnerList = new List<T>();

        protected T SearchList( string cultureID )
        {
            foreach (T item in this.InnerList)
            {
                if (item.CultureID == cultureID)
                    return item;
            }

            return default( T );
        }

        #endregion

        #region Private
        private void LoadMissingLocaleFromRepository()
        {
            IList<ILocale> list = _localeLoader.GetLocales();
            foreach (T item in list)
            {
                if (SearchList( item.CultureID ) == null)
                    this.InnerList.Add( item );
            }

            _loaded = true;
        }

        #endregion


        #region Public Constructors

        public LocaleCollection( ILocaleLoader localeLoader )
        {
            _localeLoader = localeLoader;
        }
        #endregion


        #region Public Properties
        //
        // For testing purpose
        //
        public bool Loaded
        {
            get { return _loaded; }
            set { _loaded = value; }
        }

        public T this[Culture culture]
        {
            get
            {
                return this[culture.CultureID];
            }
        }

        public T this[string cultureID]
        {
            get
            {
                T item = SearchList( cultureID );
                if (item == null)
                {
                    if (!_loaded)
                        LoadMissingLocaleFromRepository();

                    item = GetNoLoad( cultureID );
                }

                return item;
            }
        }

        #endregion


        #region Public Methods

        public T GetNoLoad( Culture culture )
        {
            return GetNoLoad( culture.CultureID );
        }

        public T GetNoLoad( string cultureID )
        {
            T item = SearchList( cultureID );

            if (item == null)
            {
                item = new T();
                item.CultureID = cultureID;

                // Do not include culture ID 0 (Culture.Null) in the locale list
                if (cultureID != "0")
                    this.InnerList.Add( item );
            }

            return item;
        }

        public void Add( T item )
        {
            if (!_loaded)
                LoadMissingLocaleFromRepository();

            if (SearchList( item.CultureID ) == null)
                this.InnerList.Add( item );
        }

        public void Load()
        {
            if (!_loaded)
                LoadMissingLocaleFromRepository();
        }

        // Deep-copy
        public object Clone()
        {
            LocaleCollection<T> newObject = new LocaleCollection<T>( _localeLoader );
            newObject._loaded = _loaded;

            foreach (T item in this.InnerList)
                newObject.InnerList.Add( (T) item.Clone() );

            return newObject;
        }

        #endregion


        #region IList<T> Members

        public int IndexOf( T item )
        {
            throw new Exception( "The method or operation is not implemented." );
        }

        public void Insert( int index, T item )
        {
            throw new Exception( "The method or operation is not implemented." );
        }

        public void RemoveAt( int index )
        {
            throw new Exception( "The method or operation is not implemented." );
        }

        public T this[int index]
        {
            get
            {
                throw new Exception( "The method or operation is not implemented." );
            }
            set
            {
                throw new Exception( "The method or operation is not implemented." );
            }
        }

        #endregion


        #region ICollection<T> Members


        public void Clear()
        {
            InnerList.Clear();
        }

        public bool Contains( T item )
        {
            Load();
            return InnerList.Contains( item );
        }

        public void CopyTo( T[] array, int arrayIndex )
        {
            throw new Exception( "The method or operation is not implemented." );
        }

        public int Count
        {
            get
            {
                Load();
                return InnerList.Count;
            }
        }

        public bool IsReadOnly
        {
            get { throw new Exception( "The method or operation is not implemented." ); }
        }

        public bool Remove( T item )
        {
            throw new Exception( "The method or operation is not implemented." );
        }

        #endregion


        #region IEnumerable<T> Members

        public IEnumerator<T> GetEnumerator()
        {
            Load();
            return InnerList.GetEnumerator();
        }

        #endregion


        #region IEnumerable Members

        IEnumerator IEnumerable.GetEnumerator()
        {
            return ((IEnumerable) InnerList).GetEnumerator();
        }

        #endregion
    }

}
