﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using LumenWorks.Framework.IO.Csv;
using Vevo.Domain.Products;
using Vevo.Shared.Domain.SystemServices;
using Vevo.Shared.SystemServices;
using Vevo.Shared.Utilities;

namespace Vevo.Domain.ImportExport
{
    public class CategoryImportHelper
    {
        #region Private
        private const string _rootID = "rootid";
        private const string _categoryID = "categoryid";
        private const string _departmentID = "departmentid";
        private const string _parentID = "parentid";
        private const string _name = "name";
        private const string _description = "description";
        private const string _image = "image";
        private const string _imageAlt = "imagealt";
        private const string _imageTitle = "imagetitle";
        private const string _isEnable = "isenabled";
        private const string _metaKeyword = "metakeyword";
        private const string _metaDescription = "metadescription";
        private const string _other1 = "other1";
        private const string _other2 = "other2";
        private const string _other3 = "other3";
        private const string _other4 = "other4";
        private const string _other5 = "other5";
        private const string _isAnchor = "isanchor";
        private const string _isShowNewArrival = "isshownewarrival";
        private const string _newArrivalAmount = "newarrivalamount";

        private IFileManager _fileManager;
        private CategoryImportHelperStatus _status;

        private DateTime _timeStart;
        private DateTime _timeEnd;

        private Culture _culture;
        private string _storeID;
        private string _tableName;

        private CsvReader CreateCsvReader( string relativePath, out TextReader reader )
        {
            Encoding encoding;
            if (_fileManager.ContainsUnicodeBom( relativePath ))
                encoding = Encoding.Unicode;
            else
                encoding = Encoding.Default;

            reader = _fileManager.CreateTextReader( relativePath, encoding );
            CsvReader csv = new CsvReader( reader, true, '\t' );

            csv.MissingFieldAction = MissingFieldAction.ReplaceByEmpty;
            csv.ParseError += ParseErrorHandler;

            return csv;
        }

        private void ParseErrorHandler( object sender, ParseErrorEventArgs e )
        {
            _status.AddError();
        }

        private Category SetupCategory( CsvReader csv, Category category, bool isParent )
        {
            category.Description = csv[_description];
            category.ImageAlt = csv[_imageAlt];
            category.ImageFile = csv[_image];
            category.ImageTitle = csv[_imageTitle];
            category.IsAnchor = ConvertUtilities.ToBoolean( csv[_isAnchor] );
            category.IsEnabled = ConvertUtilities.ToBoolean( csv[_isEnable] );
            category.IsShowNewArrival = ConvertUtilities.ToBoolean( csv[_isShowNewArrival] );
            category.MetaDescription = csv[_metaDescription];
            category.MetaKeyword = csv[_metaKeyword];
            category.Name = csv[_name];
            category.NewArrivalAmount = ConvertUtilities.ToInt32( csv[_newArrivalAmount] );
            category.Other1 = csv[_other1];
            category.Other2 = csv[_other2];
            category.Other3 = csv[_other3];
            category.Other4 = csv[_other4];
            category.Other5 = csv[_other5];
            category.CategoryID = csv[_categoryID];
            category.ParentCategoryID = csv[_parentID];
            category.RootID = csv[_rootID];

            if (isParent)
                category.ParentCategoryID = csv[_rootID];

            return category;
        }

        private Department SetupDepartment( CsvReader csv, Department department, bool isParent )
        {
            department.Description = csv[_description];
            department.ImageAlt = csv[_imageAlt];
            department.ImageFile = csv[_image];
            department.ImageTitle = csv[_imageTitle];
            department.IsAnchor = ConvertUtilities.ToBoolean( csv[_isAnchor] );
            department.IsEnabled = ConvertUtilities.ToBoolean( csv[_isEnable] );
            department.IsShowNewArrival = ConvertUtilities.ToBoolean( csv[_isShowNewArrival] );
            department.MetaDescription = csv[_metaDescription];
            department.MetaKeyword = csv[_metaKeyword];
            department.Name = csv[_name];
            department.NewArrivalAmount = ConvertUtilities.ToInt32( csv[_newArrivalAmount] );
            department.Other1 = csv[_other1];
            department.Other2 = csv[_other2];
            department.Other3 = csv[_other3];
            department.Other4 = csv[_other4];
            department.Other5 = csv[_other5];
            department.DepartmentID = csv[_departmentID];
            department.ParentDepartmentID = csv[_parentID];
            department.RootID = csv[_rootID];

            if (isParent)
                department.ParentDepartmentID = csv[_rootID];

            return department;
        }

        private bool IsCategoryFieldsValid( CsvReader csv )
        {
            bool result = false;

            if (!string.IsNullOrEmpty( csv[_categoryID] ) &&
                !string.IsNullOrEmpty( csv[_parentID] ) &&
                !string.IsNullOrEmpty( csv[_rootID] ) &&
                !string.IsNullOrEmpty( csv[_name] ))
            {
                result = true;
            }

            return result;
        }

        private bool IsDepartmentFieldsValid( CsvReader csv )
        {
            bool result = false;

            if (!string.IsNullOrEmpty( csv[_departmentID] ) &&
                !string.IsNullOrEmpty( csv[_parentID] ) &&
                !string.IsNullOrEmpty( csv[_rootID] ) &&
                !string.IsNullOrEmpty( csv[_name] ))
            {
                result = true;
            }

            return result;
        }

        private bool ImportCategoryValue( CsvReader csv, bool isUpdate, bool isParent )
        {
            bool result = false;

            Category category = new Category( _culture );
            category = SetupCategory( csv, category, isParent );
            if (isUpdate)
            {
                Category newCategory = DataAccessContext.CategoryRepository.Save( category );
            }
            else
            {
                Category newCategory = DataAccessContext.CategoryRepository.SaveWithIdentity( category );
            }

            result = true;

            return result;
        }

        private bool ImportDepartmentValue( CsvReader csv, bool isUpdate, bool isParent )
        {
            bool result = false;

            try
            {
                Department department = new Department( _culture );
                department = SetupDepartment( csv, department, isParent );
                if (isUpdate)
                {
                    Department newDepartment = DataAccessContext.DepartmentRepository.Save( department );
                }
                else
                {
                    Department newDepartment = DataAccessContext.DepartmentRepository.SaveWithIdentity( department );
                }

                result = true;
            }
            catch (Exception ex)
            {
                SaveLogFile.SaveLog( ex );
                _status.SetFatalError( ex.Message );
            }

            return result;
        }

        private bool IsCategoryExist( string categoryID )
        {
            return !DataAccessContext.CategoryRepository.GetOne( _culture, categoryID ).IsNull;
        }

        private bool IsRootCategoryExist( string rootID )
        {
            bool result = false;
            IList<Category> rootList = DataAccessContext.CategoryRepository.GetRootCategory( _culture, "", BoolFilter.ShowAll );
            foreach (Category rootCategory in rootList)
            {
                if (rootCategory.CategoryID.Equals( rootID ))
                {
                    result = true;
                    break;
                }
            }
            return result;
        }

        private bool IsDepartmentExist( string departmentID )
        {
            return !DataAccessContext.DepartmentRepository.GetOne( _culture, departmentID ).IsNull;
        }

        private bool IsRootDepartmentExist( string rootID )
        {
            bool result = false;
            IList<Department> rootList = DataAccessContext.DepartmentRepository.GetRootDepartment( _culture, "", BoolFilter.ShowAll );
            foreach (Department rootDepartment in rootList)
            {
                if (rootDepartment.DepartmentID.Equals( rootID ))
                {
                    result = true;
                    break;
                }
            }
            return result;
        }

        private CategoryImportHelperStatus ImportCategory( string filePath, out TimeSpan timeDiff )
        {
            TextReader reader = null;
            CsvReader csv = null;
            _status = new CategoryImportHelperStatus( _tableName );

            try
            {
                csv = CreateCsvReader( filePath, out reader );
                // Read data line-by-line
                int i = 2;
                while (csv.ReadNextRecord())
                {
                    if (IsCategoryFieldsValid( csv ))
                    {
                        Category category = DataAccessContext.CategoryRepository.GetOne( _culture, csv[_categoryID] );
                        if (category.IsNull)
                        {
                            if (!IsRootCategoryExist( csv[_rootID] ))
                            {
                                if (csv[_rootID].Equals( "0" ))
                                {
                                    try
                                    {
                                        if (ImportCategoryValue( csv, false, true ))
                                        {
                                            _status.AddSuccess();
                                        }
                                    }
                                    catch (Exception e)
                                    {
                                        _status.AddOtherError( i.ToString(), e );
                                    }
                                    continue;
                                }

                                _status.AddRootError( i.ToString() );
                            }
                            else
                            {
                                if (!IsCategoryExist( csv[_parentID] ))
                                {
                                    try
                                    {
                                        if (ImportCategoryValue( csv, false, true ))
                                        {
                                            _status.AddSuccess();
                                        }
                                    }
                                    catch (Exception e)
                                    {
                                        _status.AddOtherError( i.ToString(), e );
                                    }
                                }
                                else
                                {
                                    IList<Product> productList = DataAccessContext.ProductRepository.GetByCategoryID( _culture, csv[_parentID], "ProductID", BoolFilter.ShowAll, _storeID );
                                    if (productList.Count > 0)
                                    {
                                        _status.AddParentContainProductError( i.ToString() );
                                    }
                                    else
                                    {
                                        try
                                        {
                                            if (ImportCategoryValue( csv, false, false ))
                                            {
                                                _status.AddSuccess();
                                            }
                                        }
                                        catch (Exception e)
                                        {
                                            _status.AddOtherError( i.ToString(), e );
                                        }
                                    }
                                }
                            }
                        }
                        else
                        {
                            if (csv[_rootID] == category.RootID)
                            {
                                if (csv[_parentID] == category.ParentCategoryID)
                                {
                                    try
                                    {
                                        if (ImportCategoryValue( csv, true, false ))
                                        {
                                            _status.UpdateSuccess();
                                        }
                                    }
                                    catch (Exception e)
                                    {
                                        _status.AddOtherError( i.ToString(), e );
                                    }
                                }
                                else
                                {
                                    if (!IsCategoryExist( csv[_parentID] ))
                                    {
                                        try
                                        {
                                            if (ImportCategoryValue( csv, true, true ))
                                            {
                                                _status.UpdateSuccess();
                                            }
                                        }
                                        catch (Exception e)
                                        {
                                            _status.AddOtherError( i.ToString(), e );
                                        }
                                    }
                                    else
                                    {
                                        IList<Product> productList = DataAccessContext.ProductRepository.GetByCategoryID( _culture, csv[_parentID], "ProductID", BoolFilter.ShowAll, _storeID );
                                        if (productList.Count > 0)
                                        {
                                            _status.AddParentContainProductError( i.ToString() );
                                        }
                                        else
                                        {
                                            try
                                            {
                                                if (ImportCategoryValue( csv, true, false ))
                                                {
                                                    _status.UpdateSuccess();
                                                }
                                            }
                                            catch (Exception e)
                                            {
                                                _status.AddOtherError( i.ToString(), e );
                                            }
                                        }
                                    }
                                }
                            }
                            else
                            {
                                _status.AddRootChangedError( i.ToString() );
                            }
                        }
                    }
                    else
                    {
                        _status.AddFieldError( i.ToString() );
                    }

                    i++;
                }

                _status.BuildOtherError();
                _status.BuildFieldError();
                _status.BuildRootError();
                _status.BuildParentContainProductError();
                _status.BuildRootChangedError();
            }
            catch (Exception ex)
            {
                SaveLogFile.SaveLog( ex );
                _status.SetFatalError( ex.Message );
            }
            finally
            {
                csv.Dispose();
                reader.Dispose();
            }

            _timeEnd = DateTime.Now;
            timeDiff = _timeEnd.Subtract( _timeStart );
            return _status;
        }

        private CategoryImportHelperStatus ImportDepartment( string filePath, out TimeSpan timeDiff )
        {
            TextReader reader = null;
            CsvReader csv = null;
            _status = new CategoryImportHelperStatus( _tableName );

            try
            {
                csv = CreateCsvReader( filePath, out reader );
                // Read data line-by-line
                int i = 2;
                while (csv.ReadNextRecord())
                {
                    if (IsDepartmentFieldsValid( csv ))
                    {
                        Department department = DataAccessContext.DepartmentRepository.GetOne( _culture, csv[_departmentID] );
                        if (department.IsNull)
                        {
                            if (!IsRootDepartmentExist( csv[_rootID] ))
                            {
                                if (csv[_rootID].Equals( "0" ))
                                {
                                    try
                                    {
                                        if (ImportDepartmentValue( csv, false, true ))
                                        {
                                            _status.AddSuccess();
                                        }
                                    }
                                    catch (Exception e)
                                    {
                                        _status.AddOtherError( i.ToString(), e );
                                    }
                                    continue;
                                }

                                _status.AddRootError( i.ToString() );
                            }
                            else
                            {
                                if (!IsDepartmentExist( csv[_parentID] ))
                                {
                                    try
                                    {
                                        if (ImportDepartmentValue( csv, false, true ))
                                        {
                                            _status.AddSuccess();
                                        }
                                    }
                                    catch (Exception e)
                                    {
                                        _status.AddOtherError( i.ToString(), e );
                                    }
                                }
                                else
                                {
                                    IList<Product> productList = DataAccessContext.ProductRepository.GetByDepartmentID( _culture, csv[_parentID], "ProductID", BoolFilter.ShowAll, _storeID );
                                    if (productList.Count > 0)
                                    {
                                        _status.AddParentContainProductError( i.ToString() );
                                    }
                                    else
                                    {
                                        try
                                        {
                                            if (ImportDepartmentValue( csv, false, false ))
                                            {
                                                _status.AddSuccess();
                                            }
                                        }
                                        catch (Exception e)
                                        {
                                            _status.AddOtherError( i.ToString(), e );
                                        }
                                    }
                                }
                            }
                        }
                        else
                        {
                            if (csv[_rootID] == department.RootID)
                            {
                                if (csv[_parentID] == department.ParentDepartmentID)
                                {
                                    try
                                    {
                                        if (ImportDepartmentValue( csv, true, false ))
                                        {
                                            _status.UpdateSuccess();
                                        }
                                    }
                                    catch (Exception e)
                                    {
                                        _status.AddOtherError( i.ToString(), e );
                                    }
                                }
                                else
                                {
                                    if (!IsDepartmentExist( csv[_parentID] ))
                                    {
                                        try
                                        {
                                            if (ImportDepartmentValue( csv, true, true ))
                                            {
                                                _status.UpdateSuccess();
                                            }
                                        }
                                        catch (Exception e)
                                        {
                                            _status.AddOtherError( i.ToString(), e );
                                        }
                                    }
                                    else
                                    {
                                        IList<Product> productList = DataAccessContext.ProductRepository.GetByCategoryID( _culture, csv[_parentID], "ProductID", BoolFilter.ShowAll, _storeID );
                                        if (productList.Count > 0)
                                        {
                                            _status.AddParentContainProductError( i.ToString() );
                                        }
                                        else
                                        {
                                            try
                                            {
                                                if (ImportDepartmentValue( csv, true, false ))
                                                {
                                                    _status.UpdateSuccess();
                                                }
                                            }
                                            catch (Exception e)
                                            {
                                                _status.AddOtherError( i.ToString(), e );
                                            }
                                        }
                                    }
                                }
                            }
                            else
                            {
                                _status.AddRootChangedError( i.ToString() );
                            }
                        }
                    }
                    else
                    {
                        _status.AddFieldError( i.ToString() );
                    }

                    i++;
                }

                _status.BuildOtherError();
                _status.BuildFieldError();
                _status.BuildRootError();
                _status.BuildParentContainProductError();
                _status.BuildRootChangedError();
            }
            catch (Exception ex)
            {
                SaveLogFile.SaveLog( ex );
                _status.SetFatalError( ex.Message );
            }
            finally
            {
                csv.Dispose();
                reader.Dispose();
            }

            _timeEnd = DateTime.Now;
            timeDiff = _timeEnd.Subtract( _timeStart );
            return _status;
        }
        #endregion

        #region Constructors
        public CategoryImportHelper( Culture culture, IFileManager fileManager, string tableName )
        {
            _tableName = tableName;
            _culture = culture;
            _fileManager = fileManager;
        }

        public CategoryImportHelper( Culture culture, string tableName )
        {
            _tableName = tableName;
            _culture = culture;
            _fileManager = new FileManager();
        }
        #endregion

        #region Public
        public CategoryImportHelperStatus ImportCategoryOverWrite( string filePath, string storeID, out TimeSpan timeDiff )
        {
            _storeID = storeID;
            _timeStart = DateTime.Now;
            return ImportCategory( filePath, out timeDiff );
        }

        public CategoryImportHelperStatus ImportDepartmentOverWrite( string filePath, string storeID, out TimeSpan timeDiff )
        {
            _storeID = storeID;
            _timeStart = DateTime.Now;
            return ImportDepartment( filePath, out timeDiff );
        }
        #endregion
    }
}
