using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Drawing;
using System.IO;
using System.Text;
using LumenWorks.Framework.IO.Csv;
using Vevo.Domain.Exceptions;
using Vevo.Domain.Products;
using Vevo.Domain.Products.BackOffice;
using Vevo.Domain.Stores;
using Vevo.Domain.Tax;
using Vevo.Shared.Domain.SystemServices;
using Vevo.Shared.SystemServices;
using Vevo.Shared.Utilities;

namespace Vevo.Domain.ImportExport
{
    /// <summary>
    /// Summary description for ProductImporter
    /// </summary>
    public class ProductImporter
    {
        #region Private

        private Dictionary<string, string> _skuCsv;
        private Dictionary<string, string> _skuDatabase;

        private DateTime _timeStart;
        private DateTime _timeEnd;

        private Culture _culture;
        private IFileManager _fileManager;
        private IList<Category> _categoryTable;
        private IList<Department> _departmentTable;
        private ProductImportStatus _status;
        private ArrayList _importedSkuList = new ArrayList();
        private IList<TaxClass> _taxClassList;
        private IList<Store> _storeList;
        private string _storeID;
        private IList<ProductKitGroup> _productKitGroupTable;

        private bool ConvertToBool( string value )
        {
            int intValue;
            if (Int32.TryParse( value, out intValue ))
                return (intValue != 0);

            switch (value.ToLower())
            {
                case "true":
                case "yes":
                    return true;

                default:
                    return false;
            }
        }

        private double ConvertToDouble( object value )
        {
            double doubleValue;
            if (Double.TryParse( value.ToString(), out doubleValue ))
                return doubleValue;
            else
                return 0.0;
        }

        private int ConvertToInt32( object value )
        {
            int result;
            if (Int32.TryParse( value.ToString(), out result ))
                return result;
            else
                return 0;
        }

        private int ConvertToInt32DefaultValueOne( object value )
        {
            int result;
            if (Int32.TryParse( value.ToString(), out result ))
                return result;
            else
                return 1;
        }

        private decimal ConvertToDecimal( object value )
        {
            decimal result;
            if (Decimal.TryParse( value.ToString(), out result ))
                return result;
            else
                return 0;
        }

        private string ConvertDataToString( object value )
        {
            if (string.IsNullOrEmpty( value.ToString() ))
                return "0";
            else
                return value.ToString();
        }

        private bool IsRequireFieldValid( object value )
        {
            return String.IsNullOrEmpty( value.ToString() );
        }

        private bool IsSkuImported( string sku )
        {
            if (_importedSkuList.Contains( sku ))
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        private bool IsCategoryIDValid( string categoryID )
        {
            foreach (Category category in _categoryTable)
            {
                if (category.CategoryID == categoryID)
                {
                    IList<Category> categories = DataAccessContext.CategoryRepository.GetByParentID(
                        Culture.Null, categoryID, "CategoryID", BoolFilter.ShowAll );
                    if (categories.Count != 0)
                        return false;
                    else
                    {
                        if (category.RootID == "0")
                            return false;
                        else
                            return true;
                    }

                }

            }

            return false;
        }

        private bool IsDepartmentIDValid( string departmentID )
        {
            foreach (Department department in _departmentTable)
            {
                if (department.DepartmentID == departmentID)
                {
                    IList<Department> departments = DataAccessContext.DepartmentRepository.GetByParentID(
                        Culture.Null, departmentID, "DepartmentID", BoolFilter.ShowAll );
                    if (departments.Count != 0)
                        return false;
                    else
                    {
                        if (department.RootID == "0")
                            return false;
                        else
                            return true;
                    }

                }

            }

            return false;
        }

        private bool IsProductKitGroupIDValid( string productKitGroupID )
        {
            foreach (ProductKitGroup productKitGroup in _productKitGroupTable)
            {
                if (productKitGroup.ProductKitGroupID == productKitGroupID)
                {
                    return true;
                }
            }
            return false;
        }

        private bool IsCategoryExist( string categoryIDs )
        {
            string[] categoryIDsplit = ConvertToCategoryIDs( categoryIDs );

            foreach (string categoryID in categoryIDsplit)
            {
                if (!IsCategoryIDValid( categoryID ))
                    return false;
            }
            return true;
        }

        private bool IsDepartmentExist( string departmentIDs )
        {
            string[] departmentIDsplit = ConvertToCategoryIDs( departmentIDs );

            foreach (string departmentID in departmentIDsplit)
            {
                if (departmentID.Trim() != String.Empty)
                {
                    if (!IsDepartmentIDValid( departmentID ))
                        return false;
                }
            }
            return true;
        }

        private bool IsRelatedProductExists( string relatedProducts )
        {
            string[] relatedProductSplit = ConvertToCategoryIDs( relatedProducts );

            if (relatedProductSplit.Length == 0)
                return true;

            for (int i = 0; i < relatedProductSplit.Length; i++)
            {
                string productID = GetProductIDBySku( relatedProductSplit[i] );
                if (String.IsNullOrEmpty( productID ))
                {
                    if (!_skuCsv.ContainsKey( relatedProductSplit[i] ))
                    {
                        return false;
                    }
                }
            }

            return true;
        }

        private bool IsTaxClassExist( string taxClassID )
        {
            string[] ignoreList = { "", "yes", "no", "true", "false", "0" };
            foreach (string item in ignoreList)
            {
                if (taxClassID.ToLower().Trim() == item)
                    return true;
            }

            foreach (TaxClass taxClass in _taxClassList)
            {
                if (taxClass.TaxClassID == taxClassID)
                    return true;
            }
            return false;
        }

        private bool IsProductKitGroupExist( string isProductKit, string productKitGroupIDs )
        {
            if (!ConvertToBool( isProductKit ) && String.IsNullOrEmpty( productKitGroupIDs ))
                return true;

            string[] productKitGroupIDList = ConvertToCategoryIDs( productKitGroupIDs );

            foreach (string productKitGroupID in productKitGroupIDList)
            {
                if (productKitGroupID.Trim() != String.Empty)
                {
                    if (!IsProductKitGroupIDValid( productKitGroupID ))
                        return false;
                }
            }
            return true;
        }

        public Product SetProductKit( Product product, string storeID, string productKitGroupIDs )
        {
            string[] productKitGroupIDList = ConvertToCategoryIDs( productKitGroupIDs );
            IList<ProductKit> tempProductKitList = new List<ProductKit>();
            for (int i = 0; i < productKitGroupIDList.Length; i++)
            {
                ProductKit newKit = new ProductKit( _culture );
                newKit.ProductKitGroupID = productKitGroupIDList[i];
                tempProductKitList.Add( newKit );
            }

            product.ProductKits = tempProductKitList;

            return product;
        }

        private Product GetProductBySku( string sku, string storeID )
        {
            string productID = GetProductIDBySku( sku );
            if (!String.IsNullOrEmpty( productID ))
                return DataAccessContext.ProductRepository.GetOne( _culture, productID, storeID );
            else
                return new Product( _culture );
        }

        private string GetProductIDBySku( string sku )
        {
            if (_skuDatabase.ContainsKey( sku ))
            {
                return _skuDatabase[sku];
            }
            else
            {
                Product product = DataAccessContext.ProductRepository.GetOneBySku( _culture, _storeID, sku );
                if (!product.IsNull)
                    return product.ProductID;
                else
                    return String.Empty;
            }
        }

        private DateTime GetExpiredDate( string date )
        {
            if (String.IsNullOrEmpty( date ))
            {
                DateTime next7date = new DateTime( DateTime.Today.Ticks );
                next7date = next7date.AddDays( 7 );
                return Convert.ToDateTime( next7date.ToShortDateString() );
            }
            else
                return Convert.ToDateTime( date );
        }

        private string GetExpireType( string type )
        {
            if (String.Compare( type, "FixedDate", true ) == 0)
                return "FixedDate";
            else if (String.Compare( type, "RollingDate", true ) == 0)
                return "RollingDate";
            else
                return "No";
        }

        private int GetNumberOfDay( string number )
        {
            if (number == null)
                return 0;
            else
                return ConvertUtilities.ToInt32( number );
        }

        private string ConvertToTaxClassID( string value )
        {
            if (!string.IsNullOrEmpty( value ))
            {
                switch (value.ToLower())
                {
                    case "true":
                    case "yes":
                        IList<TaxClass> list = DataAccessContext.TaxClassRepository.GetAll( "TaxClassID ASC" );
                        return list[0].TaxClassID;
                    case "no":
                    case "false":
                        return TaxClass.NonTaxClassID;

                    default:
                        return value;
                }
            }
            else
                return TaxClass.NonTaxClassID;
        }

        private CsvReader CreateCsvReader( string relativePath, out TextReader reader )
        {
            Encoding encoding;
            if (_fileManager.ContainsUnicodeBom( relativePath ))
                encoding = Encoding.Unicode;
            else
                encoding = Encoding.Default;

            reader = _fileManager.CreateTextReader( relativePath, encoding );
            CsvReader csv = new CsvReader( reader, true );
            csv.MissingFieldAction = MissingFieldAction.ReplaceByEmpty;
            csv.ParseError += ParseErrorHandler;

            return csv;
        }

        private string[] ConvertToCategoryIDs( string source )
        {
            string[] strSplit = source.Split( new char[] { '|' }, StringSplitOptions.RemoveEmptyEntries );

            for (int i = 0; i < strSplit.Length; i++)
            {
                strSplit[i] = strSplit[i].Trim();
            }

            return strSplit;
        }

        private bool OverwriteProduceItem( CsvReader csv, bool processImage, string storeID, bool isSkipImage )
        {
            bool result = false;
            string sku = csv["Sku"].ToString();
            Product product = GetProductBySku( sku, storeID );

            result = SaveProductItem( product, csv, processImage, storeID, isSkipImage );

            if (!IsSkuImported( sku ))
            {
                _importedSkuList.Add( sku );
            }

            return result;
        }

        private ProductPrice SetUseAllDefaultValue( ProductPrice productPrice )
        {
            productPrice.UseDefaultPrice = true;
            productPrice.UseDefaultRetailPrice = true;
            productPrice.UseDefaultWholeSalePrice = true;
            productPrice.UseDefaultWholeSalePrice2 = true;
            productPrice.UseDefaultWholeSalePrice3 = true;

            return productPrice;
        }

        private bool IsUseDefaultValue( bool isUpdateOneStore, bool enteredUseDefault )
        {
            if (isUpdateOneStore)
                return false;
            else
                return enteredUseDefault;
        }

        private ProductPrice GetProductPrice(
            Product product, string storeID, string storeViewID, ProductPrice csvPrice, bool isUpdateSelectedStore )
        {
            bool useDefault;

            ProductPrice productPrice = product.GetProductPrice( storeID );
            productPrice.Price = ProductPrice.VerifyPrice(
                storeID,
                storeViewID,
                csvPrice.Price,
                product.GetProductPrice( Store.Null.StoreID ).Price,
                productPrice.Price,
                productPrice.UseDefaultPrice,
                IsUseDefaultValue( isUpdateSelectedStore, productPrice.UseDefaultPrice ),
                out useDefault );
            productPrice.UseDefaultPrice = useDefault;

            productPrice.RetailPrice = ProductPrice.VerifyPrice(
                storeID,
                storeViewID,
                csvPrice.RetailPrice,
                product.GetProductPrice( Store.Null.StoreID ).RetailPrice,
                productPrice.RetailPrice,
                productPrice.UseDefaultRetailPrice,
                IsUseDefaultValue( isUpdateSelectedStore, productPrice.UseDefaultRetailPrice ),
                out useDefault );
            productPrice.UseDefaultRetailPrice = useDefault;

            productPrice.WholesalePrice = ProductPrice.VerifyPrice(
                storeID,
                storeViewID,
                csvPrice.WholesalePrice,
                product.GetProductPrice( Store.Null.StoreID ).WholesalePrice,
                productPrice.WholesalePrice,
                productPrice.UseDefaultWholeSalePrice,
                IsUseDefaultValue( isUpdateSelectedStore, productPrice.UseDefaultWholeSalePrice ),
                out useDefault );
            productPrice.UseDefaultWholeSalePrice = useDefault;

            productPrice.WholesalePrice2 = ProductPrice.VerifyPrice(
               storeID,
               storeViewID,
               csvPrice.WholesalePrice2,
               product.GetProductPrice( Store.Null.StoreID ).WholesalePrice2,
               productPrice.WholesalePrice2,
               productPrice.UseDefaultWholeSalePrice2,
               IsUseDefaultValue( isUpdateSelectedStore, productPrice.UseDefaultWholeSalePrice2 ),
               out useDefault );
            productPrice.UseDefaultWholeSalePrice2 = useDefault;

            productPrice.WholesalePrice3 = ProductPrice.VerifyPrice(
              storeID,
              storeViewID,
              csvPrice.WholesalePrice3,
              product.GetProductPrice( Store.Null.StoreID ).WholesalePrice3,
              productPrice.WholesalePrice3,
              productPrice.UseDefaultWholeSalePrice3,
              IsUseDefaultValue( isUpdateSelectedStore, productPrice.UseDefaultWholeSalePrice3 ),
              out useDefault );
            productPrice.UseDefaultWholeSalePrice3 = useDefault;

            return productPrice;
        }

        private bool SaveProductItem( Product product, CsvReader csv, bool processImage, string storeID, bool isSkipImage )
        {
            string sku = csv["Sku"].ToString();

            double merchantRating = product.MerchantRating;
            string discountGroupID = String.IsNullOrEmpty( product.DiscountGroupID )
                ? "0" : product.DiscountGroupID;

            product.Name = csv["Name"];
            product.ShortDescription = csv["ShortDescription"];
            product.LongDescription = csv["LongDescription"];
            product.Sku = sku;
            product.Model = csv["Model"];
            product.Weight = ConvertUtilities.ToDouble( ConvertDataToString( csv["Weight"] ) );
            product.FixedShippingCost = false;
            product.ShippingCost = ConvertUtilities.ToDecimal( "0" );
            product.Manufacturer = csv["Manufacturer"];
            product.Keywords = csv["Keywords"];
            product.Other1 = csv["Other1"];
            product.Other2 = csv["Other2"];
            product.Other3 = csv["Other3"];
            product.Other4 = csv["Other4"];
            product.Other5 = csv["Other5"];
            product.IsTodaySpecial = ConvertToBool( csv["IsTodaySpecial"] );
            product.IsDownloadable = ConvertToBool( csv["IsDownloadable"] );
            product.DownloadPath = csv["DownloadPath"];
            product.IsEnabled = ConvertToBool( csv["IsEnabled"] );
            product.RelatedProducts = csv["RelatedProducts"];
            product.MerchantRating = merchantRating;
            product.DiscountGroupID = discountGroupID;
            product.IsFixedPrice = true;
            product.ManufacturerPartNumber = csv["ManufacturerPartNumber"];
            product.Upc = csv["Upc"];
            product.TaxClassID = ConvertToTaxClassID( csv["IsTaxable"] );
            product.UseInventory = ConvertToBool( csv["UseInventory"] );
            product.IsRecurring = ConvertToBool( csv["IsRecurring"] );
            product.IsAffiliate = ConvertToBool( csv["IsAffiliate"] );
            product.CategoryIDs = ConvertToCategoryIDs( csv["CategoryID"] );
            product.DepartmentIDs = ConvertToCategoryIDs( csv["DepartmentID"] );
            product.FreeShippingCost = ConvertToBool( csv["IsFreeShippingCost"] );
            product.MinQuantity = ConvertToInt32DefaultValueOne( csv["MinQuantity"] );
            product.MaxQuantity = ConvertToInt32( csv["MaxQuantity"] );
            product.IsCustomPrice = ConvertToBool( csv["IsCustomPrice"] );
            product.IsCallForPrice = ConvertToBool( csv["IsCallForPrice"] );
            product.Brand = csv["Brand"];
            product.ReturnTime = ConvertToInt32( csv["ReturnTime"] );
            product.IsProductKit = ConvertToBool( csv["IsProductKit"] );
            product.IsDynamicProductKitPrice = ConvertToBool( csv["IsDynamicProductKitPrice"] );
            product.IsDynamicProductKitWeight = ConvertToBool( csv["IsDynamicProductKitWeight"] );
            product.CreateDate = ConvertUtilities.ToDateTime( csv["CreateDate"] );
            product.ViewCount = ConvertToInt32( csv["ViewCount"] );

            product.Length = ConvertUtilities.ToDouble( ConvertDataToString( csv["Length"] ) );
            product.Width = ConvertUtilities.ToDouble( ConvertDataToString( csv["Width"] ) );
            product.Height = ConvertUtilities.ToDouble( ConvertDataToString( csv["Height"] ) );

            product = SetProductKit( product, storeID, csv["ProductKitGroupID"] );
            product = SetProductMetaInformation( product, storeID, csv["MetaKeyword"], csv["MetaDescription"] );

            ProductPrice csvPrice = new ProductPrice();
            csvPrice.Price = ConvertUtilities.ToDecimal( ConvertDataToString( csv["Price"] ) );
            csvPrice.RetailPrice = ConvertUtilities.ToDecimal( ConvertDataToString( csv["RetailPrice"] ) );
            csvPrice.WholesalePrice = ConvertUtilities.ToDecimal( ConvertDataToString( csv["WholesalePrice"] ) );
            csvPrice.WholesalePrice2 = ConvertUtilities.ToDecimal( ConvertDataToString( csv["WholesalePrice2"] ) );
            csvPrice.WholesalePrice3 = ConvertUtilities.ToDecimal( ConvertDataToString( csv["WholesalePrice3"] ) );
            product = SetProductPrice( product, storeID, csvPrice );

            product.ProductShippingCosts.Clear();

            if (ConvertToBool( csv["UseOption"] ))
            {
                CreateOptionAndStock( product, csv );
            }
            else
            {
                product.SetStock( ConvertUtilities.ToInt32( ConvertDataToString( csv["Stock"] ) ) );
            }

            if (processImage)
            {
                try
                {
                    product.ProductImages.Clear();
                    if (!AddProductImage( product, csv, isSkipImage ))
                    {
                        return false;
                    }
                }
                catch (Exception)
                {
                    throw;
                }
            }

            if (product.IsRecurring)
            {
                ProductRecurring recurring = new ProductRecurring();
                recurring.RecurringInterval = ConvertUtilities.ToInt32( csv["RecurringInterval"] );
                recurring.RecurringIntervalUnit = Convert.ToString( csv["RecurringIntervalUnit"] );
                recurring.RecurringNumberOfCycles = Convert.ToInt32( csv["RecurringNumberOfCycles"] );
                recurring.RecurringNumberOfTrialCycles = Convert.ToInt32( csv["RecurringNumberOfTrialCycles"] );
                recurring.RecurringTrialAmount = ConvertToDecimal( csv["RecurringTrialAmount"] );
                product.ProductRecurring = recurring;
            }
            if (product.IsCustomPrice)
            {
                ProductCustomPrice customPrice = new ProductCustomPrice();
                customPrice.DefaultPrice = ConvertUtilities.ToDecimal( csv["DefaultPrice"] );
                customPrice.MinimumPrice = ConvertUtilities.ToDecimal( csv["MinimumPrice"] );
                product.ProductCustomPrice = customPrice;
            }

            Product newProd = DataAccessContext.ProductRepository.Save( product );

            return true;
        }

        private string GetPathWithTilde( string relativePath )
        {
            if (relativePath.Trim().StartsWith( "~/" ))
                return relativePath.Trim();
            else
                return "~/" + relativePath.Trim();
        }

        private bool IsImageExists( CsvReader csv )
        {
            for (int currentIndex = 1; currentIndex <= 1; currentIndex++)
            {
                string currentimage = "LargeImage" + currentIndex.ToString();
                if (!string.IsNullOrEmpty( csv[currentimage] ))
                {
                    if (!_fileManager.Exists( GetPathWithTilde( csv[currentimage] ) ))
                    {

                        return false;
                    }
                }
            }

            return true;
        }

        private bool IsHasOptionItemInList( IList<OptionItem> optionItemLists, string optionItemID )
        {
            foreach (OptionItem item in optionItemLists)
            {
                if (item.OptionItemID == optionItemID)
                    return true;
            }
            return false;
        }

        private bool CheckOptions( CsvReader csv )
        {
            if (ConvertToBool( csv["UseOption"] ))
            {
                for (int currentIndex = 1; currentIndex <= 5; currentIndex++)
                {
                    string currentOption = "OptionGroupID" + currentIndex.ToString();
                    if (!String.IsNullOrEmpty( csv[currentOption] ))
                    {
                        IList<OptionItem> itemLists = DataAccessContext.OptionItemRepository.GetByOptionGroupID( _culture, csv[currentOption] );
                        if (itemLists.Count > 0)
                        {
                            if (ConvertToBool( csv["UseStockOption" + currentIndex.ToString()] ))
                            {
                                if (!ConvertToBool( csv["UseInventory"] ))
                                    return false;
                                if (!IsHasOptionItemInList( itemLists, String.Format( "{0}", csv["OptionItemID" + currentIndex.ToString()] ) ))
                                    return false;
                            }
                            else
                            {
                                if (!String.IsNullOrEmpty( csv["OptionItemID" + currentIndex.ToString()] ))
                                    return false;
                            }
                        }
                        else
                            return false;
                    }
                }
            }
            return true;
        }

        private bool CheckRecurringGiftDownload( CsvReader csv )
        {
            if (ConvertToBool( csv["IsRecurring"] ))
            {
                if (ConvertToBool( csv["IsGiftCertificate"] ) || ConvertToBool( csv["IsDownloadable"] ))
                    return false;
            }
            return true;
        }

        private bool CheckRecurringIntervalCycles( CsvReader csv )
        {
            if (ConvertToBool( csv["IsRecurring"] ))
            {
                if (ConvertToInt32( csv["RecurringIntervalUnit"] ) <= 0)
                    return false;
                else if (ConvertToInt32( csv["RecurringNumberOfCycles"] ) <= 0)
                    return false;
            }
            return true;
        }

        private bool CheckRecurringIntervalUnit( CsvReader csv )
        {
            if (ConvertToBool( csv["IsRecurring"] ))
            {
                string intervalUnit = csv["RecurringIntervalUnit"];
                switch (intervalUnit)
                {
                    case "Day":
                        return true;
                    case "Week":
                        return true;
                    case "Month":
                        return true;
                    case "Year":
                        return true;
                    default:
                        return false;
                }
            }
            return true;
        }

        private bool CheckMinMaxQuantity( CsvReader csv )
        {
            int minQuantity = ConvertToInt32DefaultValueOne( csv["MinQuantity"] );
            int maxQuantity = ConvertToInt32( csv["MaxQuantity"] );

            if (maxQuantity < 0 || minQuantity < 1)
            {
                return false;
            }

            if (maxQuantity != 0 && minQuantity > maxQuantity)
            {
                return false;
            }

            return true;
        }

        private bool AddProductImage( Product product, CsvReader csv, bool isSkipImage )
        {
            bool hasSecondary = false;
            for (int currentIndex = 1; currentIndex <= 10; currentIndex++)
            {
                string currentimage = "LargeImage" + currentIndex.ToString();
                if (!string.IsNullOrEmpty( csv[currentimage] ))
                {
                    string largeFilePath;
                    if (isSkipImage)
                    {
                        if (!ProductImageFile.IsImageInLargeDirectory( _fileManager, Path.GetFileName( csv[currentimage].ToString() ) ))
                        {
                            largeFilePath = ProductImageFile.CopyToLargeImageFolder( _fileManager, csv[currentimage].ToString() );
                        }
                        else
                        {
                            largeFilePath = ProductImageFile.LargeRelativeDirectory + Path.GetFileName( csv[currentimage].ToString() );
                        }
                    }
                    else
                    {
                        largeFilePath = ProductImageFile.CopyToLargeImageFolder( _fileManager, csv[currentimage].ToString() );
                    }

                    if (SaveRegularAndThumbnail( largeFilePath, product, currentIndex, csv, isSkipImage ))
                    {
                        if (hasSecondary == false)
                        {
                            if (SaveSecondaryImage( largeFilePath, product ))
                                hasSecondary = true;
                            else
                                return false;
                        }
                    }
                    else
                        return false;
                }
            }
            return true;
        }

        private Image GetImageFromLocal( string imageFilePath )
        {
            try
            {
                string path = _fileManager.GetFullPath( GetPathWithTilde( imageFilePath ) );
                using (FileStream fs = new FileStream( path, FileMode.Open, FileAccess.Read ))
                {
                    return Image.FromStream( fs );
                }
            }
            catch
            {
                return null;
            }
        }

        private bool SaveSecondaryImage( string largeFilePath, Product product )
        {
            System.Drawing.Image myImage = GetImageFromLocal( largeFilePath );

            if (myImage == null)
            {
                _status.AddImageProcessError( largeFilePath );
                return false;
            }

            try
            {
                using (ProductImageFile imageFile =
                    new ProductImageFile( _fileManager, Path.GetFileName( largeFilePath ), myImage ))
                {
                    imageFile.SaveSecondary();
                    product.ImageSecondary = imageFile.SecondaryFilePath;
                }
            }
            finally
            {
                myImage.Dispose();
            }
            return true;
        }

        private bool SaveRegularAndThumbnail(
            string largeFilePath, Product product, int currentindex, CsvReader csv, bool isSkipIfExist )
        {
            System.Drawing.Image myImage = GetImageFromLocal( largeFilePath );
            if (myImage == null)
            {
                _status.AddImageProcessError( largeFilePath );
                return false;
            }

            try
            {
                using (ProductImageFile imageFile = new ProductImageFile( _fileManager, Path.GetFileName( largeFilePath ), myImage ))
                {
                    if (isSkipIfExist)
                    {
                        if (!ProductImageFile.IsImageInRegularDirectory( _fileManager, Path.GetFileName( largeFilePath ) ))
                        {
                            imageFile.SaveRegular();
                        }
                        if (!ProductImageFile.IsImageInThumbnailDirectory( _fileManager, Path.GetFileName( largeFilePath ) ))
                        {
                            imageFile.SaveThumbnail();
                        }

                    }
                    else
                    {
                        imageFile.SaveRegular();
                        imageFile.SaveThumbnail();
                    }

                    bool isZoom = false;
                    bool isEnlarge = false;
                    if (csv["IsZoom" + currentindex.ToString()] != null)
                    {
                        isZoom = ConvertToBool( csv["IsZoom" + currentindex.ToString()] );
                    }
                    if (csv["IsEnlarge" + currentindex.ToString()] != null)
                    {
                        isEnlarge = ConvertToBool( csv["IsZoom" + currentindex.ToString()] );
                    }
                    ProductImage productImage = new ProductImage();
                    productImage.RegularImage = imageFile.RegularFilePath;
                    productImage.LargeImage = largeFilePath;
                    productImage.ThumbnailImage = imageFile.ThumbnailFilePath;
                    productImage.IsZoom = isZoom;
                    productImage.IsEnlarge = isEnlarge;
                    productImage.SortOrder = product.ProductImages.Count;

                    product.ProductImages.Add( productImage );
                }
            }
            finally
            {
                myImage.Dispose();
            }
            return true;
        }

        private void CreateOptionGroup( Product product, string optionGroupID, string useStock )
        {
            if (!String.IsNullOrEmpty( optionGroupID ))
            {
                OptionGroup optionGroup = DataAccessContext.OptionGroupRepository.GetOne(
                    _culture, optionGroupID );

                product.AddOptionGroup( optionGroup, ConvertToBool( useStock ) );
            }
        }

        private void CreateStock( Product product, string optionItemID, string stockItem, bool stockEnabled )
        {
            if (!String.IsNullOrEmpty( optionItemID ) && !String.IsNullOrEmpty( stockItem ))
            {
                string[] optionItemIDs =
                    optionItemID.Split( new char[] { ' ' }, StringSplitOptions.RemoveEmptyEntries );

                product.SetStock( ConvertUtilities.ToInt32( stockItem ), optionItemIDs );
            }
            else
            {
                product.SetStock( ConvertUtilities.ToInt32( stockItem ) );
            }
        }

        private void MakeBooleanColumnsNonNull( DataRow row )
        {
            foreach (DataColumn column in row.Table.Columns)
            {
                if (column.DataType == typeof( bool ))
                    row[column] = false;
            }
        }

        private string GetFormatOptionItemID( CsvReader csv )
        {
            return String.Format( "{0} {1} {2} {3} {4}",
                csv["OptionItemID1"],
                csv["OptionItemID2"],
                csv["OptionItemID3"],
                csv["OptionItemID4"],
                csv["OptionItemID5"] ).Trim();
        }

        private bool IsOptionGroupValid( string optionGroupID, string useStock, Product product )
        {
            if (String.IsNullOrEmpty( optionGroupID ))
                return true;

            foreach (ProductOptionGroup productOptionGroup in product.ProductOptionGroups)
            {
                if (productOptionGroup.OptionGroupID == optionGroupID
                    && productOptionGroup.UseStock == ConvertToBool( useStock ))
                {
                    return true;
                }
            }
            return false;
        }

        private void AssertOptionGroupsAreSame( Product product, CsvReader csv )
        {
            if (!IsOptionGroupValid( csv["OptionGroupID1"], csv["UseStockOption1"], product )
                || !IsOptionGroupValid( csv["OptionGroupID2"], csv["UseStockOption2"], product )
                || !IsOptionGroupValid( csv["OptionGroupID3"], csv["UseStockOption3"], product )
                || !IsOptionGroupValid( csv["OptionGroupID4"], csv["UseStockOption4"], product )
                || !IsOptionGroupValid( csv["OptionGroupID5"], csv["UseStockOption5"], product ))
            {
                throw new DomainException(
                    "OptionGroup[X]ID or UseStock[X]Index does not match with values " +
                    "of the same product on another row" );
            }
        }

        private void CreateOptionAndStock( Product product, CsvReader csv )
        {
            if (product.ProductOptionGroups.Count == 0)
            {
                CreateOptionGroup( product, csv["OptionGroupID1"], csv["UseStockOption1"] );
                CreateOptionGroup( product, csv["OptionGroupID2"], csv["UseStockOption2"] );
                CreateOptionGroup( product, csv["OptionGroupID3"], csv["UseStockOption3"] );
                CreateOptionGroup( product, csv["OptionGroupID4"], csv["UseStockOption4"] );
                CreateOptionGroup( product, csv["OptionGroupID5"], csv["UseStockOption5"] );
            }
            else
            {
                AssertOptionGroupsAreSame( product, csv );
            }

            CreateStock(
                product,
                GetFormatOptionItemID( csv ),
                ConvertDataToString( csv["Stock"] ),
                true );
        }

        private void ParseErrorHandler( object sender, ParseErrorEventArgs e )
        {
            _status.AddError();
        }

        private void LoadSkuFromCsv( string filePath )
        {
            ArrayList skuList = new ArrayList();
            TextReader reader = null;
            try
            {
                CsvReader csv = CreateCsvReader( filePath, out reader );

                _skuCsv = new Dictionary<string, string>();

                int i = 2;
                while (csv.ReadNextRecord())
                {
                    if (!_skuCsv.ContainsKey( csv["Sku"] ))
                        _skuCsv.Add( csv["Sku"], i.ToString() );

                    i++;
                }
            }
            finally
            {
                if (reader != null)
                    reader.Dispose();
            }
        }

        private ProductImportStatus ImportProduct(
            string filePath,
            bool processImage,
            bool skipImage,
            out TimeSpan timeDiff,
            string storeID
            )
        {
            TextReader reader = null;
            CsvReader csv = null;
            _status = new ProductImportStatus();
            try
            {
                LoadSkuFromCsv( filePath );

                csv = CreateCsvReader( filePath, out reader );

                // Read data line-by-line
                int i = 2;
                int invNum = 0;
                while (csv.ReadNextRecord())
                {
                    try
                    {
                        if (IsRequireFieldValid( csv["Name"] ))
                        {
                            _status.AddProductNameError( i.ToString() );
                        }
                        else if (IsRequireFieldValid( csv["Sku"] ) ||
                            (IsSkuImported( csv["Sku"] ) && !ConvertToBool( csv["UseOption"] )))
                        {
                            _status.AddSKUImportedError( i.ToString() );
                        }
                        else if (IsRequireFieldValid( csv["UseInventory"] ) ||
                            !Int32.TryParse( csv["Stock"].ToString(), out invNum ) ||
                            (ConvertToBool( csv["UseInventory"] )) && IsRequireFieldValid( csv["Stock"] ))
                        {
                            _status.AddInventoryAndStockError( i.ToString() );
                        }
                        else if (IsRequireFieldValid( csv["CategoryID"] ) || !IsCategoryExist( csv["CategoryID"] ))
                        {
                            _status.AddCategoryError( i.ToString() );
                        }
                        else if (!IsDepartmentExist( csv["DepartmentID"] ))
                        {
                            _status.AddDepartmentError( i.ToString() );
                        }
                        else if (!IsRelatedProductExists( csv["RelatedProducts"] ))
                        {
                            _status.AddRelatedProductError( i.ToString() );
                        }
                        else if (processImage && !IsImageExists( csv ))
                        {
                            _status.AddImageUploadError( i.ToString() );
                        }
                        else if (!CheckOptions( csv ))
                        {
                            _status.AddUseOptionError( i.ToString() );
                        }
                        else if (!CheckRecurringGiftDownload( csv ))
                        {
                            _status.AddRecurringGiftDownloadError( i.ToString() );
                        }
                        else if (!CheckRecurringIntervalCycles( csv ))
                        {
                            _status.AddRecurringIntervalCyclesError( i.ToString() );
                        }
                        else if (!CheckRecurringIntervalUnit( csv ))
                        {
                            _status.AddRecurringIntervalUnitError( i.ToString() );
                        }
                        else if (!IsTaxClassExist( csv["IsTaxable"] ))
                        {
                            _status.AddTaxClassError( i.ToString() );
                        }
                        else if (!CheckMinMaxQuantity( csv ))
                        {
                            _status.AddMinMaxQuantityError( i.ToString() );
                        }
                        else if (!IsProductKitGroupExist( csv["IsProductKit"], csv["ProductKitGroupID"] ))
                        {
                            _status.AddProductKitGroupError( i.ToString() );
                        }
                        else
                        {
                            if (OverwriteProduceItem( csv, processImage, storeID, skipImage ))
                            {
                                _status.AddSuccess();
                            }
                        }
                    }
                    catch (Exception ex)
                    {
                        //SaveLogFile.SaveLog( ex );
                        _status.AddOtherError( i.ToString(), ex );
                    }
                    i++;
                }

                try
                {
                    UpdateRelatedProductsFromIDFromSkuToID( storeID );
                }
                catch (Exception ex)
                {
                    //SaveLogFile.SaveLog( ex );
                    _status.SetFatalError( "Related Product - " + ex.Message );
                }
                _status.BuildSKUImportedError();
                _status.BuildProductNameError();
                _status.BuildRelatedProductError();
                _status.BuildCategoryError();
                _status.BuildDepartmentError();
                _status.BuildImageUploadError();
                _status.BuildOtherError();
                _status.BuildUseOptionError();
                _status.BuildImageProcessError();
                _status.BuildRecurringGiftDownloadError();
                _status.BuildRecurringIntervalCyclesError();
                _status.BuildRecurringIntervalUnitError();
                _status.BuildTaxClassError();
                _status.BuildMinMaxQuantityError();
                _status.BuildProductKitGroupError();
                _status.BuildInventoryAndStockError();
            }
            catch (Exception ex)
            {
                SaveLogFile.SaveLog( ex );
                _status.SetFatalError( ex.Message );
            }
            finally
            {
                if (csv != null)
                    csv.Dispose();

                if (reader != null)
                    reader.Dispose();
            }

            _timeEnd = DateTime.Now;
            timeDiff = _timeEnd.Subtract( _timeStart );
            return _status;
        }

        private void UpdateRelatedProductsFromIDFromSkuToID( string storeID )
        {
            foreach (Object sku in _importedSkuList)
            {
                int errorCount = 0;
                string productID = String.Empty;
                ArrayList relatedArray = new ArrayList();

                productID = GetProductIDBySku( sku.ToString() );
                if (!String.IsNullOrEmpty( productID ) && int.Parse( productID ) > 0)
                {
                    Product product = DataAccessContext.ProductRepository.GetOne(
                        _culture, productID, storeID );
                    string[] relatedProductSku = GetArrayOfRelatedProducts( product.RelatedProducts );
                    if ((relatedProductSku.Length == 1) && (relatedProductSku[0] == ""))
                    {
                    }
                    else
                    {
                        for (int i = 0; i < relatedProductSku.Length; i++)
                        {
                            string relatedID = GetProductIDBySku( relatedProductSku[i].Trim() );

                            if (!String.IsNullOrEmpty( relatedID ) && (relatedID != productID))
                            {
                                if (relatedArray.Count == 0)
                                    relatedArray.Add( relatedID );
                                else
                                {
                                    bool haveSameID = CheckSameRelatedProductID( relatedArray, relatedID );

                                    if (!haveSameID)
                                        relatedArray.Add( relatedID );
                                    else
                                        errorCount++;
                                }
                            }
                            else
                                errorCount++;
                        }
                        product.RelatedProducts = GetRelatedProductsID( relatedArray );
                        DataAccessContext.ProductRepository.Save( product );
                    }
                }
            }
        }

        private bool CheckSameRelatedProductID( ArrayList relatedArray, string relatedProductID )
        {
            int countSameRelatedID = 0;
            foreach (Object relatedID in relatedArray)
            {
                if (relatedID.ToString() == relatedProductID)
                    countSameRelatedID++;
            }

            if (countSameRelatedID > 0)
                return true;
            else
                return false;
        }

        public string GetRelatedProductsID( ArrayList relatedArray )
        {
            int i = 0;
            int count = relatedArray.Count;
            string relatedProductIDs = String.Empty;

            if (relatedArray == null)
                return relatedProductIDs;
            else
            {
                foreach (Object productID in relatedArray)
                {
                    if (i != (count - 1))
                        relatedProductIDs += (productID.ToString() + ", ");
                    else
                        relatedProductIDs += productID.ToString();
                    i++;
                }
                return relatedProductIDs;
            }
        }

        private string[] GetArrayOfRelatedProducts( string relatedProducts )
        {
            return relatedProducts.Split( '|' );
        }

        #endregion


        #region Constructors

        public ProductImporter( Culture culture, IFileManager fileManager )
        {
            _culture = culture;
            _fileManager = fileManager;
            _categoryTable = DataAccessContext.CategoryRepository.GetAll( culture, "CategoryID" );
            _departmentTable = DataAccessContext.DepartmentRepository.GetAll( culture, "DepartmentID" );
            _taxClassList = DataAccessContext.TaxClassRepository.GetAll( "TaxClassID" );
            _storeList = DataAccessContext.StoreRepository.GetAll( "StoreID" );
            _productKitGroupTable = DataAccessContext.ProductKitGroupRepository.GetAll( culture, "ProductKitGroupID" );
        }

        public ProductImporter( Culture culture )
        {
            _culture = culture;
            _fileManager = new FileManager();
            _categoryTable = DataAccessContext.CategoryRepository.GetAll( culture, "CategoryID" );
            _departmentTable = DataAccessContext.DepartmentRepository.GetAll( culture, "DepartmentID" );
            _taxClassList = DataAccessContext.TaxClassRepository.GetAll( "TaxClassID" );
            _storeList = DataAccessContext.StoreRepository.GetAll( "StoreID" );
            _productKitGroupTable = DataAccessContext.ProductKitGroupRepository.GetAll( culture, "ProductKitGroupID" );
        }

        #endregion


        #region Public Methods

        public ProductImportStatus ImportProductPurgeAll(
            string filePath, bool processImage, bool skipImage, out TimeSpan timeDiff, string storeID )
        {
            _timeStart = DateTime.Now;
            _storeID = storeID;
            DataAccessContext.ProductRepository.DeleteAll();
            _skuDatabase = new Dictionary<string, string>();
            return ImportProduct( filePath, processImage, skipImage, out timeDiff, storeID );
        }

        public ProductImportStatus ImportProductOverwrite(
            string filePath, bool processImage, bool skipImage, out TimeSpan timeDiff, string storeID )
        {
            _timeStart = DateTime.Now;
            _storeID = storeID;
            _skuDatabase = DataAccessContext.ProductRepository.GetProductIDAndSku();
            return ImportProduct( filePath, processImage, skipImage, out timeDiff, storeID );
        }

        public Product SetProductMetaInformation( Product product, string storeID, string metaKeyword, string metaDescription )
        {
            if (product.GetMetaInformation( "0" ) == null)
            {
                // Purge
                product.SetMetaKeyword( "0", metaKeyword );
                product.SetMetaDescription( "0", metaDescription );
                foreach (Store store in _storeList)
                {
                    product.SetMetaKeyword( store.StoreID, metaKeyword );
                    product.SetMetaDescription( store.StoreID, metaDescription );
                }
            }
            else
            {
                // Overwrite
                if (storeID == "0")
                {
                    product.SetMetaKeyword( "0", metaKeyword );
                    product.SetMetaDescription( "0", metaDescription );
                    foreach (Store store in _storeList)
                    {
                        ProductMetaInformation meta = product.GetMetaInformation( store.StoreID );
                        if (meta == null)
                        {
                            product.SetMetaKeyword( store.StoreID, metaKeyword );
                            product.SetMetaDescription( store.StoreID, metaDescription );
                        }
                        else
                        {
                            if (meta.UseDefaultMetaKeyword == true)
                            {
                                product.SetMetaKeyword( store.StoreID, metaKeyword );
                            }
                            if (meta.UseDefaultMetaDescription == true)
                            {
                                product.SetMetaDescription( store.StoreID, metaDescription );
                            }
                        }
                    }
                }
                else
                {
                    product.SetUseDefaultValueMetaKeyword( storeID, false );
                    product.SetUseDefaultValueMetaDescription( storeID, false );
                    product.SetMetaKeyword( storeID, metaKeyword );
                    product.SetMetaDescription( storeID, metaDescription );
                }
            }

            return product;
        }

        public Product SetProductPrice( Product product, string storeID, ProductPrice csvPrice )
        {
            bool isNewProductPrice;
            ProductPrice defaultPrice = product.GetProductPrice( Store.Null.StoreID, out isNewProductPrice );

            if (isNewProductPrice)
            {
                defaultPrice.Price = csvPrice.Price;
                defaultPrice.RetailPrice = csvPrice.RetailPrice;
                defaultPrice.WholesalePrice = csvPrice.WholesalePrice;
                defaultPrice.WholesalePrice2 = csvPrice.WholesalePrice2;
                defaultPrice.WholesalePrice3 = csvPrice.WholesalePrice3;
                defaultPrice = SetUseAllDefaultValue( defaultPrice );

                product.SetPrice( Store.Null.StoreID, defaultPrice );

                foreach (Store store in _storeList)
                {
                    product.SetPrice( store.StoreID, defaultPrice );
                }
            }
            else
            {
                ProductPrice productPrice = GetProductPrice( product, storeID, storeID, csvPrice, true );

                if (storeID != "0")
                {
                    product.SetPrice( storeID, productPrice );
                }
                else
                {
                    productPrice = SetUseAllDefaultValue( productPrice );
                    product.SetPrice( storeID, productPrice );

                    foreach (Store store in _storeList)
                    {
                        product.SetPrice(
                            store.StoreID,
                            GetProductPrice( product, store.StoreID, Store.Null.StoreID, csvPrice, false ) );
                    }
                }
            }

            return product;
        }

        #endregion
    }
}